@@ -1,6 +1,6 @@
 /*  vcfroh.c -- HMM model for detecting runs of autozygosity.
 
-    Copyright (C) 2013-2020 Genome Research Ltd.
+    Copyright (C) 2013-2022 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -153,7 +153,7 @@ static void init_data(args_t *args)
         args->pl_hdr_id = bcf_hdr_id2int(args->hdr, BCF_DT_ID, "PL");
         if ( !bcf_hdr_idinfo_exists(args->hdr,BCF_HL_FMT,args->pl_hdr_id) )
             error("Error: The FORMAT/PL tag not found in the header, consider running with -G\n");
-        if ( bcf_hdr_id2type(args->hdr,BCF_HL_FMT,args->pl_hdr_id)!=BCF_HT_INT ) 
+        if ( bcf_hdr_id2type(args->hdr,BCF_HL_FMT,args->pl_hdr_id)!=BCF_HT_INT )
             error("Error: The FORMAT/PL tag not defined as Integer in the header\n");
     }
 
@@ -254,7 +254,7 @@ static void init_data(args_t *args)
         {
             if ( *end!=',') error("Could not parse: --buffer-size %s\n", args->buffer_size);
             args->nbuf_olap = strtol(end+1,&end,10);
-            if ( *end || args->nbuf_olap<0 ) error("Could not parse: --bufer-size %s\n", args->buffer_size);
+            if ( *end || args->nbuf_olap<0 ) error("Could not parse: --buffer-size %s\n", args->buffer_size);
         }
         if ( tmp<0 )
             args->nbuf_max = fabs(tmp)*1e6/(4+8*2)/args->roh_smpl->n;
@@ -279,15 +279,15 @@ static void init_data(args_t *args)
     MAT(tprob,2,STATE_HW,STATE_HW) = 1 - args->t2AZ;
     MAT(tprob,2,STATE_HW,STATE_AZ) = args->t2HW;
     MAT(tprob,2,STATE_AZ,STATE_HW) = args->t2AZ;
-    MAT(tprob,2,STATE_AZ,STATE_AZ) = 1 - args->t2HW; 
+    MAT(tprob,2,STATE_AZ,STATE_AZ) = 1 - args->t2HW;
 
     args->hmm = hmm_init(2, tprob, 10000);
-    if ( args->genmap_fname ) 
+    if ( args->genmap_fname )
         hmm_set_tprob_func(args->hmm, set_tprob_genmap, args);
     else if ( args->rec_rate > 0 )
         hmm_set_tprob_func(args->hmm, set_tprob_rrate, args);
 
-    args->out = bgzf_open(strcmp("stdout",args->output_fname)?args->output_fname:"-", args->output_type&OUTPUT_GZ ? "wg" : "wu"); 
+    args->out = bgzf_open(strcmp("stdout",args->output_fname)?args->output_fname:"-", args->output_type&OUTPUT_GZ ? "wg" : "wu");
     if ( !args->out ) error("Failed to open %s: %s\n", args->output_fname, strerror(errno));
 
     // print header
@@ -509,7 +509,7 @@ static void flush_viterbi(args_t *args, int ismpl)
 
     if ( !args->vi_training ) // single viterbi pass
     {
-        hmm_restore(args->hmm, smpl->snapshot); 
+        hmm_restore(args->hmm, smpl->snapshot);
         int end = (args->nbuf_max && smpl->nsites >= args->nbuf_max && smpl->nsites > args->nbuf_olap) ? smpl->nsites - args->nbuf_olap : smpl->nsites;
         if ( end < smpl->nsites )
             smpl->snapshot = hmm_snapshot(args->hmm, smpl->snapshot, smpl->sites[smpl->nsites - args->nbuf_olap - 1]);
@@ -535,7 +535,7 @@ static void flush_viterbi(args_t *args, int ismpl)
 
             if ( args->output_type & OUTPUT_RG )
             {
-                if ( state!=smpl->rg.state ) 
+                if ( state!=smpl->rg.state )
                 {
                     if ( !state )   // the region ends, flush
                     {
@@ -599,7 +599,7 @@ static void flush_viterbi(args_t *args, int ismpl)
     MAT(tprob_arr,2,STATE_HW,STATE_HW) = 1 - args->t2AZ;
     MAT(tprob_arr,2,STATE_HW,STATE_AZ) = args->t2HW;
     MAT(tprob_arr,2,STATE_AZ,STATE_HW) = args->t2AZ;
-    MAT(tprob_arr,2,STATE_AZ,STATE_AZ) = 1 - args->t2HW; 
+    MAT(tprob_arr,2,STATE_AZ,STATE_AZ) = 1 - args->t2HW;
     hmm_set_tprob(args->hmm, tprob_arr, 10000);
 
     int niter = 0;
@@ -627,14 +627,14 @@ static void flush_viterbi(args_t *args, int ismpl)
         delthw = fabs(MAT(tprob_new,2,STATE_HW,STATE_AZ)-t2hw_prev);
         niter++;
         args->str.l = 0;
-        ksprintf(&args->str, "VT\t%s\t%d\t%e\t%e\t%e\t%e\t%e\t%e\n", 
+        ksprintf(&args->str, "VT\t%s\t%d\t%e\t%e\t%e\t%e\t%e\t%e\n",
             name,niter,deltaz,delthw,
             1-MAT(tprob_new,2,STATE_HW,STATE_HW),MAT(tprob_new,2,STATE_AZ,STATE_HW),
             1-MAT(tprob_new,2,STATE_AZ,STATE_AZ),MAT(tprob_new,2,STATE_HW,STATE_AZ));
         if ( bgzf_write(args->out, args->str.s, args->str.l) != args->str.l ) error("Error writing %s: %s\n", args->output_fname, strerror(errno));
     }
     while ( deltaz > args->baum_welch_th || delthw > args->baum_welch_th );
-    
+
     // output the results
     for (i=0; i<smpl->nrid; i++)
     {
@@ -660,6 +660,8 @@ static void flush_viterbi(args_t *args, int ismpl)
 
 int read_AF(bcf_sr_regions_t *tgt, bcf1_t *line, double *alt_freq)
 {
+    if ( tgt->nals < 2 )
+        error("Expected two comma-separated alleles (REF,ALT) in the third column of %s, found:\n\t%s\n", tgt->fname,tgt->line.s);
     if ( tgt->nals != line->n_allele ) return -1;    // number of alleles does not match
 
     int i;
@@ -669,7 +671,7 @@ int read_AF(bcf_sr_regions_t *tgt, bcf1_t *line, double *alt_freq)
 
     char *tmp, *str = tgt->line.s;
     i = 0;
-    while ( *str && i<3 ) 
+    while ( *str && i<3 )
     {
         if ( *str=='\t' ) i++;
         str++;
@@ -720,7 +722,11 @@ int estimate_AF_from_GT(args_t *args, int8_t *gt, double *alt_freq)
         int8_t *end = gt + 2*bcf_hdr_nsamples(args->hdr);
         while ( gt < end )
         {
-            if ( bcf_gt_is_missing(gt[0]) || bcf_gt_is_missing(gt[1]) ) continue;
+            if ( bcf_gt_is_missing(gt[0]) || bcf_gt_is_missing(gt[1]) )
+            {
+                gt += 2;
+                continue;
+            }
 
             if ( bcf_gt_allele(gt[0]) ) nalt++;
             else nref++;
@@ -744,7 +750,7 @@ int estimate_AF_from_PL(args_t *args, bcf_fmt_t *fmt_pl, int ial, double *alt_fr
 
     int irr = bcf_alleles2gt(0,0), ira = bcf_alleles2gt(0,ial), iaa = bcf_alleles2gt(ial,ial);
     if ( iaa >= fmt_pl->n ) return -1;  // not diploid or wrong number of fields
-    
+
     if ( args->af_smpl )        // subset samples for AF estimate
     {
         #define BRANCH(type_t) \
@@ -836,7 +842,7 @@ int process_line(args_t *args, bcf1_t *line, int ial)
         if ( ret==-2 )
             error("Type mismatch for INFO/%s tag at %s:%"PRId64"\n", args->af_tag, bcf_seqname(args->hdr,line), (int64_t) line->pos+1);
     }
-    else if ( args->af_fname ) 
+    else if ( args->af_fname )
     {
         // Read AF from a file
         ret = read_AF(args->files->targets, line, &alt_freq);
@@ -873,9 +879,9 @@ int process_line(args_t *args, bcf1_t *line, int ial)
             if ( ret>0 )
                 AC = args->itmp[0];
         }
-        if ( AN<=0 || AC<0 ) 
+        if ( AN<=0 || AC<0 )
             ret = -1;
-        else 
+        else
             alt_freq = (double) AC/AN;
     }
 
@@ -960,12 +966,12 @@ int process_line(args_t *args, bcf1_t *line, int ial)
             smpl->eprob = (double*) realloc(smpl->eprob,sizeof(*smpl->eprob)*smpl->msites*2);
             if ( !smpl->eprob ) error("Error: failed to alloc %"PRIu64" bytes\n", (uint64_t)(sizeof(*smpl->eprob)*smpl->msites*2));
         }
-        
+
         // Calculate emission probabilities P(D|AZ) and P(D|HW)
         double *eprob = &smpl->eprob[2*smpl->nsites];
         eprob[STATE_AZ] = pdg[0]*(1-alt_freq) + pdg[2]*alt_freq;
         eprob[STATE_HW] = pdg[0]*(1-alt_freq)*(1-alt_freq) + 2*pdg[1]*(1-alt_freq)*alt_freq + pdg[2]*alt_freq*alt_freq;
-        
+
         smpl->sites[smpl->nsites] = line->pos;
         smpl->nsites++;
 
@@ -992,12 +998,12 @@ static void vcfroh(args_t *args, bcf1_t *line)
 
     // Are we done?
     if ( !line )
-    { 
+    {
         for (i=0; i<args->roh_smpl->n; i++) flush_viterbi(args, i);
-        return; 
+        return;
     }
 
-    // Skip unwanted lines, for simplicity we consider only biallelic sites 
+    // Skip unwanted lines, for simplicity we consider only biallelic sites
     if ( line->rid == args->skip_rid ) return;
 
     // This can be raw callable VCF with the symbolic unseen allele <*>
@@ -1041,7 +1047,7 @@ static void vcfroh(args_t *args, bcf1_t *line)
         args->prev_pos = line->pos;
         skip_rid = load_genmap(args, bcf_seqname(args->hdr,line));
     }
-    else if ( args->prev_pos == line->pos ) 
+    else if ( args->prev_pos == line->pos )
     {
         args->ndup++;
         return;     // skip duplicate positions
@@ -1092,10 +1098,12 @@ static void usage(args_t *args)
     fprintf(stderr, "    -O, --output-type [srz]            output s:per-site, r:regions, z:compressed [sr]\n");
     fprintf(stderr, "    -r, --regions <region>             restrict to comma-separated list of regions\n");
     fprintf(stderr, "    -R, --regions-file <file>          restrict to regions listed in a file\n");
+    fprintf(stderr, "        --regions-overlap 0|1|2        Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n");
     fprintf(stderr, "    -s, --samples <list>               list of samples to analyze [all samples]\n");
     fprintf(stderr, "    -S, --samples-file <file>          file of samples to analyze [all samples]\n");
     fprintf(stderr, "    -t, --targets <region>             similar to -r but streams rather than index-jumps\n");
     fprintf(stderr, "    -T, --targets-file <file>          similar to -R but streams rather than index-jumps\n");
+    fprintf(stderr, "        --targets-overlap 0|1|2        Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n");
     fprintf(stderr, "        --threads <int>                use multithreading with <int> worker threads [0]\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "HMM Options:\n");
@@ -1116,6 +1124,8 @@ int main_vcfroh(int argc, char *argv[])
     args->t2HW    = 5e-9;
     args->rec_rate = 0;
     int regions_is_file = 0, targets_is_file = 0;
+    int regions_overlap = 1;
+    int targets_overlap = 0;
 
     static struct option loptions[] =
     {
@@ -1138,8 +1148,10 @@ int main_vcfroh(int argc, char *argv[])
         {"viterbi-training",1,0,'V'},
         {"targets",1,0,'t'},
         {"targets-file",1,0,'T'},
+        {"targets-overlap",required_argument,NULL,6},
         {"regions",1,0,'r'},
         {"regions-file",1,0,'R'},
+        {"regions-overlap",required_argument,NULL,7},
         {"genetic-map",1,0,'m'},
         {"rec-rate",1,0,'M'},
         {"skip-indels",0,0,'I'},
@@ -1153,15 +1165,19 @@ int main_vcfroh(int argc, char *argv[])
         switch (c) {
             case 0: args->af_tag = optarg; naf_opts++; break;
             case 1: args->af_fname = optarg; naf_opts++; break;
-            case 2: 
+            case 2:
                 args->dflt_AF = strtod(optarg,&tmp);
                 if ( *tmp ) error("Could not parse: --AF-dflt %s\n", optarg);
                 break;
-            case 3: args->filter_str = optarg; args->filter_logic = FLT_INCLUDE; break;
-            case 4: args->filter_str = optarg; args->filter_logic = FLT_EXCLUDE; break;
+            case  3 :
+                if ( args->filter_str ) error("Error: only one --include or --exclude expression can be given, and they cannot be combined\n");
+                args->filter_str = optarg; args->filter_logic |= FLT_INCLUDE; break;
+            case  4 :
+                if ( args->filter_str ) error("Error: only one --include or --exclude expression can be given, and they cannot be combined\n");
+                args->filter_str = optarg; args->filter_logic |= FLT_EXCLUDE; break;
             case 5: args->include_noalt_sites = 1; break;
             case 'o': args->output_fname = optarg; break;
-            case 'O': 
+            case 'O':
                 if ( strchr(optarg,'s') || strchr(optarg,'S') ) args->output_type |= OUTPUT_ST;
                 if ( strchr(optarg,'r') || strchr(optarg,'R') ) args->output_type |= OUTPUT_RG;
                 if ( strchr(optarg,'z') || strchr(optarg,'z') ) args->output_type |= OUTPUT_GZ;
@@ -1171,10 +1187,10 @@ int main_vcfroh(int argc, char *argv[])
             case 'i': args->skip_homref = 1; break;
             case 'I': args->snps_only = 1; break;
             case 'G':
-                args->fake_PLs = 1; 
+                args->fake_PLs = 1;
                 args->unseen_PL = strtod(optarg,&tmp);
                 if ( *tmp ) error("Could not parse: -G %s\n", optarg);
-                args->unseen_PL = pow(10,-args->unseen_PL/10.); 
+                args->unseen_PL = pow(10,-args->unseen_PL/10.);
                 break;
             case 'm': args->genmap_fname = optarg; break;
             case 'M':
@@ -1195,13 +1211,21 @@ int main_vcfroh(int argc, char *argv[])
             case 'T': args->targets_list = optarg; targets_is_file = 1; break;
             case 'r': args->regions_list = optarg; break;
             case 'R': args->regions_list = optarg; regions_is_file = 1; break;
+            case  6 :
+                targets_overlap = parse_overlap_option(optarg);
+                if ( targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
+            case  7 :
+                regions_overlap = parse_overlap_option(optarg);
+                if ( regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
-            case 'V': 
-                args->vi_training = 1; 
-                args->baum_welch_th = strtod(optarg,&tmp); 
+            case 'V':
+                args->vi_training = 1;
+                args->baum_welch_th = strtod(optarg,&tmp);
                 if ( *tmp ) error("Could not parse: --viterbi-training %s\n", optarg);
                 break;
-            case 'h': 
+            case 'h':
             case '?': usage(args); break;
             default: error("Unknown argument: %s\n", optarg);
         }
@@ -1223,11 +1247,13 @@ int main_vcfroh(int argc, char *argv[])
     if ( args->af_fname && args->targets_list ) error("Error: The options --AF-file and -t are mutually exclusive\n");
     if ( args->regions_list )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_REGIONS_OVERLAP,regions_overlap);
         if ( bcf_sr_set_regions(args->files, args->regions_list, regions_is_file)<0 )
             error("Failed to read the regions: %s\n", args->regions_list);
     }
     if ( args->targets_list )
     {
+        bcf_sr_set_opt(args->files,BCF_SR_TARGETS_OVERLAP,targets_overlap);
         if ( bcf_sr_set_targets(args->files, args->targets_list, targets_is_file, 0)<0 )
             error("Failed to read the targets: %s\n", args->targets_list);
     }
